@@ -1,7 +1,7 @@
 /*
  * cryptsetup kernel RNG access functions
  *
- * Copyright (C) 2010-2019 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2010-2022 Red Hat, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,7 +20,6 @@
 
 #include <stdlib.h>
 #include <string.h>
-#include <fcntl.h>
 #include <errno.h>
 #include <assert.h>
 #include <sys/select.h>
@@ -28,10 +27,6 @@
 #include "libcryptsetup.h"
 #include "internal.h"
 
-#ifndef O_CLOEXEC
-#define O_CLOEXEC 0
-#endif
-
 static int random_initialised = 0;
 
 #define URANDOM_DEVICE	"/dev/urandom"
@@ -47,8 +42,7 @@ static int random_fd = -1;
 #define RANDOM_DEVICE_TIMEOUT	5
 
 /* URANDOM_DEVICE access */
-static int _get_urandom(struct crypt_device *ctx __attribute__((unused)),
-			char *buf, size_t len)
+static int _get_urandom(char *buf, size_t len)
 {
 	int r;
 	size_t old_len = len;
@@ -56,7 +50,7 @@ static int _get_urandom(struct crypt_device *ctx __attribute__((unused)),
 
 	assert(urandom_fd != -1);
 
-	while(len) {
+	while (len) {
 		r = read(urandom_fd, buf, len);
 		if (r == -1 && errno != EINTR)
 			return -EINVAL;
@@ -158,20 +152,20 @@ int crypt_random_init(struct crypt_device *ctx)
 	if(urandom_fd == -1)
 		urandom_fd = open(URANDOM_DEVICE, O_RDONLY | O_CLOEXEC);
 	if(urandom_fd == -1)
-		goto fail;
+		goto err;
 
 	/* Used for CRYPT_RND_KEY */
 	if(random_fd == -1)
 		random_fd = open(RANDOM_DEVICE, O_RDONLY | O_NONBLOCK | O_CLOEXEC);
 	if(random_fd == -1)
-		goto fail;
+		goto err;
 
 	if (crypt_fips_mode())
 		log_verbose(ctx, _("Running in FIPS mode."));
 
 	random_initialised = 1;
 	return 0;
-fail:
+err:
 	crypt_random_exit();
 	log_err(ctx, _("Fatal error during RNG initialisation."));
 	return -ENOSYS;
@@ -183,13 +177,13 @@ int crypt_random_get(struct crypt_device *ctx, char *buf, size_t len, int qualit
 
 	switch(quality) {
 	case CRYPT_RND_NORMAL:
-		status = _get_urandom(ctx, buf, len);
+		status = _get_urandom(buf, len);
 		break;
 	case CRYPT_RND_SALT:
 		if (crypt_fips_mode())
 			status = crypt_backend_rng(buf, len, quality, 1);
 		else
-			status = _get_urandom(ctx, buf, len);
+			status = _get_urandom(buf, len);
 		break;
 	case CRYPT_RND_KEY:
 		if (crypt_fips_mode()) {
@@ -200,7 +194,7 @@ int crypt_random_get(struct crypt_device *ctx, char *buf, size_t len, int qualit
 				 crypt_random_default_key_rng();
 		switch (rng_type) {
 		case CRYPT_RNG_URANDOM:
-			status = _get_urandom(ctx, buf, len);
+			status = _get_urandom(buf, len);
 			break;
 		case CRYPT_RNG_RANDOM:
 			status = _get_random(ctx, buf, len);
