@@ -1,7 +1,7 @@
 /*
  * dm-verity volume handling
  *
- * Copyright (C) 2012-2020 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2012-2022 Red Hat, Inc. All rights reserved.
  *
  * This file is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -23,10 +23,9 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdint.h>
+#include <ctype.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <fcntl.h>
-#include <netinet/in.h>
 #include <uuid/uuid.h>
 
 #include "libcryptsetup.h"
@@ -88,8 +87,7 @@ int VERITY_read_sb(struct crypt_device *cd,
 		return -EIO;
 
 	if (memcmp(sb.signature, VERITY_SIGNATURE, sizeof(sb.signature))) {
-		log_err(cd, _("Device %s is not a valid VERITY device."),
-			device_path(device));
+		log_dbg(cd, "No VERITY signature detected.");
 		return -EINVAL;
 	}
 
@@ -113,6 +111,10 @@ int VERITY_read_sb(struct crypt_device *cd,
 	}
 	params->data_size = le64_to_cpu(sb.data_blocks);
 
+	/* Update block size to be used for loop devices */
+	device_set_block_size(crypt_metadata_device(cd), params->hash_block_size);
+	device_set_block_size(crypt_data_device(cd), params->data_block_size);
+
 	params->hash_name = strndup((const char*)sb.algorithm, sizeof(sb.algorithm));
 	if (!params->hash_name)
 		return -ENOMEM;
@@ -146,6 +148,13 @@ int VERITY_read_sb(struct crypt_device *cd,
 	return 0;
 }
 
+static void _to_lower(char *str)
+{
+	for(; *str; str++)
+		if (isupper(*str))
+			*str = tolower(*str);
+}
+
 /* Write verity superblock to disk */
 int VERITY_write_sb(struct crypt_device *cd,
 		   uint64_t sb_offset,
@@ -155,6 +164,7 @@ int VERITY_write_sb(struct crypt_device *cd,
 	struct device *device = crypt_metadata_device(cd);
 	struct verity_sb sb = {};
 	ssize_t hdr_size = sizeof(struct verity_sb);
+	size_t block_size;
 	char *algorithm;
 	uuid_t uuid;
 	int r, devfd;
@@ -174,6 +184,13 @@ int VERITY_write_sb(struct crypt_device *cd,
 		return -EINVAL;
 	}
 
+	/* Avoid possible increasing of image size - FEC could fail later because of it */
+	block_size = device_block_size(cd, device);
+	if (block_size > params->hash_block_size) {
+		device_disable_direct_io(device);
+		block_size = params->hash_block_size;
+	}
+
 	devfd = device_open(cd, device, O_RDWR);
 	if (devfd < 0) {
 		log_err(cd, _("Cannot open device %s."), device_path(device));
@@ -187,13 +204,17 @@ int VERITY_write_sb(struct crypt_device *cd,
 	sb.hash_block_size = cpu_to_le32(params->hash_block_size);
 	sb.salt_size       = cpu_to_le16(params->salt_size);
 	sb.data_blocks     = cpu_to_le64(params->data_size);
+
+	/* Kernel always use lower-case */
 	algorithm = (char *)sb.algorithm;
-	algorithm[sizeof(sb.algorithm)-1] = '\0';
 	strncpy(algorithm, params->hash_name, sizeof(sb.algorithm)-1);
+	algorithm[sizeof(sb.algorithm)-1] = '\0';
+	_to_lower(algorithm);
+
 	memcpy(sb.salt, params->salt, params->salt_size);
 	memcpy(sb.uuid, uuid, sizeof(sb.uuid));
 
-	r = write_lseek_blockwise(devfd, device_block_size(cd, device), device_alignment(device),
+	r = write_lseek_blockwise(devfd, block_size, device_alignment(device),
 				  (char*)&sb, hdr_size, sb_offset) < hdr_size ? -EIO : 0;
 	if (r)
 		log_err(cd, _("Error during update of verity header on device %s."),
@@ -218,7 +239,7 @@ uint64_t VERITY_hash_offset_block(struct crypt_params_verity *params)
 	return hash_offset / params->hash_block_size;
 }
 
-int VERITY_UUID_generate(struct crypt_device *cd, char **uuid_string)
+int VERITY_UUID_generate(char **uuid_string)
 {
 	uuid_t uuid;
 
@@ -242,7 +263,7 @@ int VERITY_activate(struct crypt_device *cd,
 {
 	uint32_t dmv_flags;
 	unsigned int fec_errors = 0;
-	int r;
+	int r, v;
 	struct crypt_dm_active_device dmd = {
 		.size = verity_hdr->data_size * verity_hdr->data_block_size / 512,
 		.flags = activation_flags,
@@ -261,14 +282,19 @@ int VERITY_activate(struct crypt_device *cd,
 		log_dbg(cd, "Verification of data in userspace required.");
 		r = VERITY_verify(cd, verity_hdr, root_hash, root_hash_size);
 
-		if (r == -EPERM && fec_device) {
+		if ((r == -EPERM || r == -EFAULT) && fec_device) {
+			v = r;
 			log_dbg(cd, "Verification failed, trying to repair with FEC device.");
 			r = VERITY_FEC_process(cd, verity_hdr, fec_device, 1, &fec_errors);
 			if (r < 0)
 				log_err(cd, _("Errors cannot be repaired with FEC device."));
-			else if (fec_errors)
+			else if (fec_errors) {
 				log_err(cd, _("Found %u repairable errors with FEC device."),
 					fec_errors);
+				/* If root hash failed, we cannot be sure it was properly repaired */
+			}
+			if (v == -EFAULT)
+				r = -EPERM;
 		}
 
 		if (r < 0)
@@ -299,7 +325,7 @@ int VERITY_activate(struct crypt_device *cd,
 			crypt_metadata_device(cd), fec_device, root_hash,
 			root_hash_size, signature_description,
 			VERITY_hash_offset_block(verity_hdr),
-			VERITY_hash_blocks(cd, verity_hdr), verity_hdr);
+			VERITY_FEC_blocks(cd, fec_device, verity_hdr), verity_hdr);
 
 	if (r)
 		return r;
@@ -328,3 +354,63 @@ out:
 	dm_targets_free(cd, &dmd);
 	return r;
 }
+
+int VERITY_dump(struct crypt_device *cd,
+		struct crypt_params_verity *verity_hdr,
+		const char *root_hash,
+		unsigned int root_hash_size,
+		struct device *fec_device)
+{
+	uint64_t hash_blocks, verity_blocks, fec_blocks = 0, rs_blocks = 0;
+	bool fec_on_hash_device = false;
+
+	hash_blocks  = VERITY_hash_blocks(cd, verity_hdr);
+	verity_blocks = VERITY_hash_offset_block(verity_hdr) + hash_blocks;
+
+	if (fec_device && verity_hdr->fec_roots) {
+		fec_blocks = VERITY_FEC_blocks(cd, fec_device, verity_hdr);
+		rs_blocks  = VERITY_FEC_RS_blocks(fec_blocks, verity_hdr->fec_roots);
+		fec_on_hash_device = device_is_identical(crypt_metadata_device(cd), fec_device) > 0;
+		/*
+		* No way to access fec_area_offset directly.
+		* Assume FEC area starts directly after hash blocks.
+		*/
+		if (fec_on_hash_device)
+			verity_blocks += rs_blocks;
+	}
+
+	log_std(cd, "VERITY header information for %s\n", device_path(crypt_metadata_device(cd)));
+	log_std(cd, "UUID:            \t%s\n", crypt_get_uuid(cd) ?: "");
+	log_std(cd, "Hash type:       \t%u\n", verity_hdr->hash_type);
+	log_std(cd, "Data blocks:     \t%" PRIu64 "\n", verity_hdr->data_size);
+	log_std(cd, "Data block size: \t%u\n", verity_hdr->data_block_size);
+	log_std(cd, "Hash blocks:     \t%" PRIu64 "\n", hash_blocks);
+	log_std(cd, "Hash block size: \t%u\n", verity_hdr->hash_block_size);
+	log_std(cd, "Hash algorithm:  \t%s\n", verity_hdr->hash_name);
+	if (fec_device && fec_blocks) {
+		log_std(cd, "FEC RS roots:   \t%" PRIu32 "\n", verity_hdr->fec_roots);
+		log_std(cd, "FEC blocks:     \t%" PRIu64 "\n", rs_blocks);
+	}
+
+	log_std(cd, "Salt:            \t");
+	if (verity_hdr->salt_size)
+		crypt_log_hex(cd, verity_hdr->salt, verity_hdr->salt_size, "", 0, NULL);
+	else
+		log_std(cd, "-");
+	log_std(cd, "\n");
+
+	if (root_hash) {
+		log_std(cd, "Root hash:      \t");
+		crypt_log_hex(cd, root_hash, root_hash_size, "", 0, NULL);
+		log_std(cd, "\n");
+	}
+
+	/* As dump can take only hash device, we have no idea about offsets here. */
+	if (verity_hdr->hash_area_offset == 0)
+		log_std(cd, "Hash device size: \t%" PRIu64 " [bytes]\n", verity_blocks * verity_hdr->hash_block_size);
+
+	if (fec_device && verity_hdr->fec_area_offset == 0 && fec_blocks && !fec_on_hash_device)
+		log_std(cd, "FEC device size: \t%" PRIu64 " [bytes]\n", rs_blocks * verity_hdr->data_block_size);
+
+	return 0;
+}
