@@ -3,8 +3,8 @@
  *
  * Copyright (C) 2004 Jana Saout <jana@saout.de>
  * Copyright (C) 2004-2007 Clemens Fruhwirth <clemens@endorphin.org>
- * Copyright (C) 2009-2019 Red Hat, Inc. All rights reserved.
- * Copyright (C) 2009-2019 Milan Broz
+ * Copyright (C) 2009-2022 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2009-2022 Milan Broz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,6 +24,7 @@
 #ifndef CRYPTSETUP_H
 #define CRYPTSETUP_H
 
+#include <stdbool.h>
 #include <string.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -40,32 +41,22 @@
 #include <sys/time.h>
 
 #include "lib/nls.h"
+#include "lib/bitops.h"
 #include "lib/utils_crypt.h"
 #include "lib/utils_loop.h"
 #include "lib/utils_fips.h"
 #include "lib/utils_io.h"
 #include "lib/utils_blkid.h"
+#include "lib/libcryptsetup_macros.h"
 
 #include "libcryptsetup.h"
 
-#define CONST_CAST(x) (x)(uintptr_t)
 #define DEFAULT_CIPHER(type)	(DEFAULT_##type##_CIPHER "-" DEFAULT_##type##_MODE)
-#define SECTOR_SIZE 512
-#define MAX_SECTOR_SIZE 4096
-#define ROUND_SECTOR(x) (((x) + SECTOR_SIZE - 1) / SECTOR_SIZE)
 
 #define DEFAULT_WIPE_BLOCK	1048576 /* 1 MiB */
-
-extern int opt_debug;
-extern int opt_debug_json;
-extern int opt_verbose;
-extern int opt_batch_mode;
-extern int opt_force_password;
-extern int opt_progress_frequency;
+#define MAX_ACTIONS 16
 
 /* Common tools */
-void clogger(struct crypt_device *cd, int level, const char *file, int line,
-	     const char *format, ...)  __attribute__ ((format (printf, 5, 6)));
 void tool_log(int level, const char *msg, void *usrptr __attribute__((unused)));
 void quiet_log(int level, const char *msg, void *usrptr);
 
@@ -81,6 +72,8 @@ int translate_errno(int r);
 typedef enum { CREATED, UNLOCKED, REMOVED  } crypt_object_op;
 void tools_keyslot_msg(int keyslot, crypt_object_op op);
 void tools_token_msg(int token, crypt_object_op op);
+void tools_token_error_msg(int error, const char *type, int token, bool pin_provided);
+void tools_package_version(const char *name, bool use_pwlibs);
 
 extern volatile int quit;
 void set_int_block(int block);
@@ -96,30 +89,90 @@ int tools_get_key(const char *prompt,
 		  struct crypt_device *cd);
 void tools_passphrase_msg(int r);
 int tools_is_stdin(const char *key_file);
-int tools_string_to_size(struct crypt_device *cd, const char *s, uint64_t *size);
-int tools_is_cipher_null(const char *cipher);
+int tools_string_to_size(const char *s, uint64_t *size);
 
-void tools_clear_line(void);
+struct tools_progress_params {
+	uint32_t frequency;
+	struct timeval start_time;
+	struct timeval end_time;
+	uint64_t start_offset;
+	bool batch_mode;
+	bool json_output;
+	const char *interrupt_message;
+	const char *device;
+};
 
-int tools_wipe_progress(uint64_t size, uint64_t offset, void *usrptr);
-int tools_reencrypt_progress(uint64_t size, uint64_t offset, void *usrptr);
+int tools_progress(uint64_t size, uint64_t offset, void *usrptr);
+const char *tools_get_device_name(const char *device, char **r_backing_file);
 
-int tools_read_mk(const char *file, char **key, int keysize);
+int tools_read_vk(const char *file, char **key, int keysize);
 int tools_write_mk(const char *file, const char *key, int keysize);
 
-int tools_read_json_file(struct crypt_device *cd, const char *file, char **json, size_t *json_size);
-int tools_write_json_file(struct crypt_device *cd, const char *file, const char *json);
+int tools_read_json_file(const char *file, char **json, size_t *json_size, bool batch_mode);
+int tools_write_json_file(const char *file, const char *json);
 
-int tools_detect_signatures(const char *device, int ignore_luks, size_t *count);
-int tools_wipe_all_signatures(const char *path);
+typedef enum {
+	PRB_FILTER_NONE = 0,
+	PRB_FILTER_LUKS,
+	PRB_ONLY_LUKS
+} tools_probe_filter_info;
+
+int tools_detect_signatures(const char *device, tools_probe_filter_info filter, size_t *count, bool batch_mode);
+int tools_wipe_all_signatures(const char *path, bool exclusive, bool only_luks);
+int tools_superblock_block_size(const char *device, char *sb_name,
+				size_t sb_name_len, unsigned *r_block_size);
+bool tools_blkid_supported(void);
 
 int tools_lookup_crypt_device(struct crypt_device *cd, const char *type,
-		const char *data_device_path, char *name, size_t name_length);
+		const char *data_device_path, char **r_name);
+
+
+/* each utility is required to implement it */
+void tools_cleanup(void);
 
 /* Log */
-#define log_dbg(x...) clogger(NULL, CRYPT_LOG_DEBUG, __FILE__, __LINE__, x)
-#define log_std(x...) clogger(NULL, CRYPT_LOG_NORMAL, __FILE__, __LINE__, x)
-#define log_verbose(x...) clogger(NULL, CRYPT_LOG_VERBOSE, __FILE__, __LINE__, x)
-#define log_err(x...) clogger(NULL, CRYPT_LOG_ERROR, __FILE__, __LINE__, x)
+#define log_dbg(x...) crypt_logf(NULL, CRYPT_LOG_DEBUG, x)
+#define log_std(x...) crypt_logf(NULL, CRYPT_LOG_NORMAL, x)
+#define log_verbose(x...) crypt_logf(NULL, CRYPT_LOG_VERBOSE, x)
+#define log_err(x...) crypt_logf(NULL, CRYPT_LOG_ERROR, x)
+
+typedef enum {
+	CRYPT_ARG_BOOL = 0,
+	CRYPT_ARG_STRING,
+	CRYPT_ARG_INT32,
+	CRYPT_ARG_UINT32,
+	CRYPT_ARG_INT64,
+	CRYPT_ARG_UINT64,
+	CRYPT_ARG_ALIAS
+} crypt_arg_type_info;
+
+struct tools_arg {
+	const char *name;
+	bool set;
+	crypt_arg_type_info type;
+	union {
+		char *str_value;
+		uint64_t u64_value;
+		uint32_t u32_value;
+		int32_t i32_value;
+		int64_t i64_value;
+		union {
+			unsigned id;
+			struct tools_arg *ptr;
+		} o;
+	} u;
+	const char *actions_array[MAX_ACTIONS];
+};
+
+void tools_parse_arg_value(poptContext popt_context, crypt_arg_type_info type, struct tools_arg *arg, const char *popt_arg, int popt_val, bool(*needs_size_conv_fn)(unsigned arg_id));
+
+void tools_args_free(struct tools_arg *args, size_t args_count);
+
+void tools_check_args(const char *action, const struct tools_arg *args, size_t args_size, poptContext popt_context);
+
+struct tools_log_params {
+	bool verbose;
+	bool debug;
+};
 
 #endif /* CRYPTSETUP_H */
