@@ -1,8 +1,8 @@
 /*
  * veritysetup - setup cryptographic volumes for dm-verity
  *
- * Copyright (C) 2012-2019 Red Hat, Inc. All rights reserved.
- * Copyright (C) 2012-2019 Milan Broz
+ * Copyright (C) 2012-2022 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2012-2022 Milan Broz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,29 +20,18 @@
  */
 
 #include "cryptsetup.h"
+#include "veritysetup_args.h"
 
 #define PACKAGE_VERITY "veritysetup"
 
-static int use_superblock = 1;
-
-static const char *fec_device = NULL;
-static int fec_roots = DEFAULT_VERITY_FEC_ROOTS;
-static const char *hash_algorithm = NULL;
-static int hash_type = 1;
-static int data_block_size = DEFAULT_VERITY_DATA_BLOCK;
-static int hash_block_size = DEFAULT_VERITY_HASH_BLOCK;
-static uint64_t data_blocks = 0;
-static const char *salt_string = NULL;
-static uint64_t hash_offset = 0;
-static uint64_t fec_offset = 0;
-static const char *opt_uuid = NULL;
-static int opt_restart_on_corruption = 0;
-static int opt_ignore_corruption = 0;
-static int opt_ignore_zero_blocks = 0;
-static int opt_check_at_most_once = 0;
-
 static const char **action_argv;
 static int action_argc;
+static struct tools_log_params log_parms;
+
+void tools_cleanup(void)
+{
+	tools_args_free(tool_core_args, ARRAY_SIZE(tool_core_args));
+}
 
 static int _prepare_format(struct crypt_params_verity *params,
 			   const char *data_device,
@@ -51,16 +40,16 @@ static int _prepare_format(struct crypt_params_verity *params,
 	char *salt = NULL;
 	int len;
 
-	params->hash_name = hash_algorithm ?: DEFAULT_VERITY_HASH;
+	params->hash_name = ARG_STR(OPT_HASH_ID);
 	params->data_device = data_device;
-	params->fec_device = fec_device;
-	params->fec_roots = fec_roots;
+	params->fec_device = ARG_STR(OPT_FEC_DEVICE_ID);
+	params->fec_roots = ARG_UINT32(OPT_FEC_ROOTS_ID);
 
-	if (salt_string && !strcmp(salt_string, "-")) {
+	if (ARG_STR(OPT_SALT_ID) && !strcmp(ARG_STR(OPT_SALT_ID), "-")) {
 		params->salt_size = 0;
 		params->salt = NULL;
-	} else if (salt_string) {
-		len = crypt_hex_to_bytes(salt_string, &salt, 0);
+	} else if (ARG_SET(OPT_SALT_ID)) {
+		len = crypt_hex_to_bytes(ARG_STR(OPT_SALT_ID), &salt, 0);
 		if (len < 0) {
 			log_err(_("Invalid salt string specified."));
 			return -EINVAL;
@@ -72,23 +61,25 @@ static int _prepare_format(struct crypt_params_verity *params,
 		params->salt = NULL;
 	}
 
-	params->data_block_size = data_block_size;
-	params->hash_block_size = hash_block_size;
-	params->data_size = data_blocks;
-	params->hash_area_offset = hash_offset;
-	params->fec_area_offset = fec_offset;
-	params->hash_type = hash_type;
+	params->data_block_size = ARG_UINT32(OPT_DATA_BLOCK_SIZE_ID);
+	params->hash_block_size = ARG_UINT32(OPT_HASH_BLOCK_SIZE_ID);
+	params->data_size = ARG_UINT64(OPT_DATA_BLOCKS_ID);
+	params->hash_area_offset = ARG_UINT64(OPT_HASH_OFFSET_ID);
+	params->fec_area_offset = ARG_UINT64(OPT_FEC_OFFSET_ID);
+	params->hash_type = ARG_UINT32(OPT_FORMAT_ID);
 	params->flags = flags;
 
 	return 0;
 }
 
-static int action_format(int arg)
+static int action_format(void)
 {
 	struct crypt_device *cd = NULL;
 	struct crypt_params_verity params = {};
 	uint32_t flags = CRYPT_VERITY_CREATE_HASH;
-	int r;
+	char *root_hash_bytes = NULL;
+	size_t root_hash_size;
+	int root_hash_fd = -1, i, r;
 
 	/* Try to create hash image if doesn't exist */
 	r = open(action_argv[1], O_WRONLY | O_EXCL | O_CREAT, S_IRUSR | S_IWUSR);
@@ -100,13 +91,13 @@ static int action_format(int arg)
 		close(r);
 	}
 	/* Try to create FEC image if doesn't exist */
-	if (fec_device) {
-		r = open(fec_device, O_WRONLY | O_EXCL | O_CREAT, S_IRUSR | S_IWUSR);
+	if (ARG_SET(OPT_FEC_DEVICE_ID)) {
+		r = open(ARG_STR(OPT_FEC_DEVICE_ID), O_WRONLY | O_EXCL | O_CREAT, S_IRUSR | S_IWUSR);
 		if (r < 0 && errno != EEXIST) {
-			log_err(_("Cannot create FEC image %s for writing."), fec_device);
+			log_err(_("Cannot create FEC image %s for writing."), ARG_STR(OPT_FEC_DEVICE_ID));
 			return -EINVAL;
 		} else if (r >= 0) {
-			log_dbg("Created FEC image %s.", fec_device);
+			log_dbg("Created FEC image %s.", ARG_STR(OPT_FEC_DEVICE_ID));
 			close(r);
 		}
 	}
@@ -114,19 +105,54 @@ static int action_format(int arg)
 	if ((r = crypt_init(&cd, action_argv[1])))
 		goto out;
 
-	if (!use_superblock)
+	if (ARG_SET(OPT_NO_SUPERBLOCK_ID))
 		flags |= CRYPT_VERITY_NO_HEADER;
 
 	r = _prepare_format(&params, action_argv[0], flags);
 	if (r < 0)
 		goto out;
 
-	r = crypt_format(cd, CRYPT_VERITY, NULL, NULL, opt_uuid, NULL, 0, &params);
-	if (!r)
-		crypt_dump(cd);
+	r = crypt_format(cd, CRYPT_VERITY, NULL, NULL, ARG_STR(OPT_UUID_ID), NULL, 0, &params);
+	if (r < 0)
+		goto out;
+
+	crypt_dump(cd);
+
+	/* Create or overwrite the root hash file */
+	if (ARG_SET(OPT_ROOT_HASH_FILE_ID)) {
+		root_hash_size = crypt_get_volume_key_size(cd);
+		root_hash_bytes = malloc(root_hash_size);
+		if (!root_hash_bytes) {
+			r = -ENOMEM;
+			goto out;
+		}
+
+		r = crypt_volume_key_get(cd, CRYPT_ANY_SLOT, root_hash_bytes, &root_hash_size, NULL, 0);
+		if (r < 0)
+			goto out;
+
+		root_hash_fd = open(ARG_STR(OPT_ROOT_HASH_FILE_ID), O_WRONLY | O_TRUNC | O_CREAT, S_IRUSR | S_IWUSR);
+		if (root_hash_fd == -1) {
+			log_err(_("Cannot create root hash file %s for writing."), ARG_STR(OPT_ROOT_HASH_FILE_ID));
+			r = -EINVAL;
+			goto out;
+		}
+
+		for (i = 0; i < (int)root_hash_size; i++)
+			if (dprintf(root_hash_fd, "%02hhx", root_hash_bytes[i]) != 2) {
+				log_err(_("Cannot write to root hash file %s."), ARG_STR(OPT_ROOT_HASH_FILE_ID));
+				r = -EIO;
+				goto out;
+			}
+
+		log_dbg("Created root hash file %s.", ARG_STR(OPT_ROOT_HASH_FILE_ID));
+	}
 out:
 	crypt_free(cd);
 	free(CONST_CAST(char*)params.salt);
+	free(root_hash_bytes);
+	if (root_hash_fd != -1)
+		close(root_hash_fd);
 	return r;
 }
 
@@ -139,29 +165,36 @@ static int _activate(const char *dm_device,
 	struct crypt_device *cd = NULL;
 	struct crypt_params_verity params = {};
 	uint32_t activate_flags = CRYPT_ACTIVATE_READONLY;
-	char *root_hash_bytes = NULL;
-	ssize_t hash_size;
-	int r;
+	char *root_hash_bytes = NULL, *root_hash_from_file = NULL;
+	ssize_t hash_size, hash_size_hex;
+	struct stat st;
+	char *signature = NULL;
+	int signature_size = 0, root_hash_fd = -1, r;
 
 	if ((r = crypt_init_data_device(&cd, hash_device, data_device)))
 		goto out;
 
-	if (opt_ignore_corruption)
+	if (ARG_SET(OPT_IGNORE_CORRUPTION_ID))
 		activate_flags |= CRYPT_ACTIVATE_IGNORE_CORRUPTION;
-	if (opt_restart_on_corruption)
+	if (ARG_SET(OPT_RESTART_ON_CORRUPTION_ID))
 		activate_flags |= CRYPT_ACTIVATE_RESTART_ON_CORRUPTION;
-	if (opt_ignore_zero_blocks)
+	if (ARG_SET(OPT_PANIC_ON_CORRUPTION_ID))
+		activate_flags |= CRYPT_ACTIVATE_PANIC_ON_CORRUPTION;
+	if (ARG_SET(OPT_IGNORE_ZERO_BLOCKS_ID))
 		activate_flags |= CRYPT_ACTIVATE_IGNORE_ZERO_BLOCKS;
-	if (opt_check_at_most_once)
+	if (ARG_SET(OPT_CHECK_AT_MOST_ONCE_ID))
 		activate_flags |= CRYPT_ACTIVATE_CHECK_AT_MOST_ONCE;
 
-	if (use_superblock) {
+	if (!ARG_SET(OPT_NO_SUPERBLOCK_ID)) {
 		params.flags = flags;
-		params.hash_area_offset = hash_offset;
-		params.fec_area_offset = fec_offset;
-		params.fec_device = fec_device;
-		params.fec_roots = fec_roots;
+		params.hash_area_offset = ARG_UINT64(OPT_HASH_OFFSET_ID);
+		params.fec_area_offset = ARG_UINT64(OPT_FEC_OFFSET_ID);
+		params.fec_device = ARG_STR(OPT_FEC_DEVICE_ID);
+		params.fec_roots = ARG_UINT32(OPT_FEC_ROOTS_ID);
 		r = crypt_load(cd, CRYPT_VERITY, &params);
+		if (r)
+			log_err(_("Device %s is not a valid VERITY device."), hash_device);
+
 	} else {
 		r = _prepare_format(&params, data_device, flags | CRYPT_VERITY_NO_HEADER);
 		if (r < 0)
@@ -172,61 +205,137 @@ static int _activate(const char *dm_device,
 		goto out;
 
 	hash_size = crypt_get_volume_key_size(cd);
+	hash_size_hex = 2 * hash_size;
+
+	if (!root_hash) {
+		root_hash_fd = open(ARG_STR(OPT_ROOT_HASH_FILE_ID), O_RDONLY);
+		if (root_hash_fd == -1) {
+			log_err(_("Cannot read root hash file %s."), ARG_STR(OPT_ROOT_HASH_FILE_ID));
+			goto out;
+		}
+
+		if (fstat(root_hash_fd, &st) || !S_ISREG(st.st_mode) || st.st_size < hash_size_hex) {
+			log_err(_("Invalid root hash file %s."), ARG_STR(OPT_ROOT_HASH_FILE_ID));
+			r = -EINVAL;
+			goto out;
+		}
+
+		root_hash_from_file = malloc(hash_size_hex + 1);
+		if (!root_hash_from_file) {
+			r = -ENOMEM;
+			goto out;
+		}
+
+		if (read_buffer(root_hash_fd, root_hash_from_file, hash_size_hex) != hash_size_hex) {
+			log_err(_("Cannot read root hash file %s."), root_hash_from_file);
+			goto out;
+		}
+
+		root_hash_from_file[hash_size_hex] = '\0';
+		root_hash = root_hash_from_file;
+	}
+
 	if (crypt_hex_to_bytes(root_hash, &root_hash_bytes, 0) != hash_size) {
 		log_err(_("Invalid root hash string specified."));
 		r = -EINVAL;
 		goto out;
 	}
-	r = crypt_activate_by_volume_key(cd, dm_device,
+
+	if (ARG_SET(OPT_ROOT_HASH_SIGNATURE_ID)) {
+		// FIXME: check max file size
+		if (stat(ARG_STR(OPT_ROOT_HASH_SIGNATURE_ID), &st) || !S_ISREG(st.st_mode) || !st.st_size) {
+			log_err(_("Invalid signature file %s."), ARG_STR(OPT_ROOT_HASH_SIGNATURE_ID));
+			r = -EINVAL;
+			goto out;
+		}
+		signature_size = st.st_size;
+		r = tools_read_vk(ARG_STR(OPT_ROOT_HASH_SIGNATURE_ID), &signature, signature_size);
+		if (r < 0) {
+			log_err(_("Cannot read signature file %s."), ARG_STR(OPT_ROOT_HASH_SIGNATURE_ID));
+			goto out;
+		}
+	}
+	r = crypt_activate_by_signed_key(cd, dm_device,
 					 root_hash_bytes,
 					 hash_size,
+					 signature, signature_size,
 					 activate_flags);
 out:
+	crypt_safe_free(signature);
 	crypt_free(cd);
+	free(root_hash_from_file);
 	free(root_hash_bytes);
 	free(CONST_CAST(char*)params.salt);
+	if (root_hash_fd != -1)
+		close(root_hash_fd);
 	return r;
 }
 
-static int action_open(int arg)
+static int action_open(void)
 {
+	if (action_argc < 4 && !ARG_SET(OPT_ROOT_HASH_FILE_ID)) {
+		log_err(_("Command requires <root_hash> or --root-hash-file option as argument."));
+		return -EINVAL;
+	}
+
 	return _activate(action_argv[1],
 			 action_argv[0],
 			 action_argv[2],
-			 action_argv[3], 0);
+			 ARG_SET(OPT_ROOT_HASH_FILE_ID) ? NULL : action_argv[3],
+			 ARG_SET(OPT_ROOT_HASH_SIGNATURE_ID) ? CRYPT_VERITY_ROOT_HASH_SIGNATURE : 0);
 }
 
-static int action_verify(int arg)
+static int action_verify(void)
 {
+	if (action_argc < 3 && !ARG_SET(OPT_ROOT_HASH_FILE_ID)) {
+		log_err(_("Command requires <root_hash> or --root-hash-file option as argument."));
+		return -EINVAL;
+	}
+
 	return _activate(NULL,
 			 action_argv[0],
 			 action_argv[1],
-			 action_argv[2],
+			 ARG_SET(OPT_ROOT_HASH_FILE_ID) ? NULL : action_argv[2],
 			 CRYPT_VERITY_CHECK_HASH);
 }
 
-static int action_close(int arg)
+static int action_close(void)
 {
 	struct crypt_device *cd = NULL;
+	crypt_status_info ci;
+	uint32_t flags = 0;
 	int r;
 
+	if (ARG_SET(OPT_DEFERRED_ID))
+		flags |= CRYPT_DEACTIVATE_DEFERRED;
+	if (ARG_SET(OPT_CANCEL_DEFERRED_ID))
+		flags |= CRYPT_DEACTIVATE_DEFERRED_CANCEL;
+
 	r = crypt_init_by_name(&cd, action_argv[0]);
 	if (r == 0)
-		r = crypt_deactivate(cd, action_argv[0]);
+		r = crypt_deactivate_by_name(cd, action_argv[0], flags);
+
+	if (!r && ARG_SET(OPT_DEFERRED_ID)) {
+		ci = crypt_status(cd, action_argv[0]);
+		if (ci == CRYPT_ACTIVE || ci == CRYPT_BUSY)
+			log_std(_("Device %s is still active and scheduled for deferred removal.\n"),
+				  action_argv[0]);
+	}
 
 	crypt_free(cd);
 	return r;
 }
 
-static int action_status(int arg)
+static int action_status(void)
 {
 	crypt_status_info ci;
 	struct crypt_active_device cad;
 	struct crypt_params_verity vp = {};
 	struct crypt_device *cd = NULL;
 	struct stat st;
-	char *backing_file;
-	unsigned i, path = 0;
+	char *backing_file, *root_hash;
+	size_t root_hash_size;
+	unsigned path = 0;
 	int r = 0;
 
 	/* perhaps a path, not a dm device name */
@@ -269,8 +378,9 @@ static int action_status(int arg)
 		if (r < 0)
 			goto out;
 
-		log_std("  status:      %s\n",
-			cad.flags & CRYPT_ACTIVATE_CORRUPTED ? "corrupted" : "verified");
+		log_std("  status:      %s%s\n",
+			cad.flags & CRYPT_ACTIVATE_CORRUPTED ? "corrupted" : "verified",
+			vp.flags & CRYPT_VERITY_ROOT_HASH_SIGNATURE ? " (with signature)" : "");
 
 		log_std("  hash type:   %u\n", vp.hash_type);
 		log_std("  data block:  %u\n", vp.data_block_size);
@@ -278,8 +388,7 @@ static int action_status(int arg)
 		log_std("  hash name:   %s\n", vp.hash_name);
 		log_std("  salt:        ");
 		if (vp.salt_size)
-			for(i = 0; i < vp.salt_size; i++)
-				log_std("%02hhx", (const char)vp.salt[i]);
+			crypt_log_hex(NULL, vp.salt, vp.salt_size, "", 0, NULL);
 		else
 			log_std("-");
 		log_std("\n");
@@ -303,7 +412,7 @@ static int action_status(int arg)
 
 		if (vp.fec_device) {
 			log_std("  FEC device:  %s\n", vp.fec_device);
-			if ((backing_file = crypt_loop_backing_file(vp.fec_device))) {
+			if ((backing_file = crypt_loop_backing_file(ARG_STR(OPT_FEC_DEVICE_ID)))) {
 				log_std("  FEC loop:    %s\n", backing_file);
 				free(backing_file);
 			}
@@ -311,13 +420,27 @@ static int action_status(int arg)
 				vp.fec_area_offset * vp.hash_block_size / 512);
 			log_std("  FEC roots:   %u\n", vp.fec_roots);
 		}
+
+		root_hash_size = crypt_get_volume_key_size(cd);
+		if (root_hash_size > 0 && (root_hash = malloc(root_hash_size))) {
+			r = crypt_volume_key_get(cd, CRYPT_ANY_SLOT, root_hash, &root_hash_size, NULL, 0);
+			if (!r) {
+				log_std("  root hash:   ");
+				crypt_log_hex(NULL, root_hash, root_hash_size, "", 0, NULL);
+				log_std("\n");
+			}
+			free(root_hash);
+		}
+
 		if (cad.flags & (CRYPT_ACTIVATE_IGNORE_CORRUPTION|
 				 CRYPT_ACTIVATE_RESTART_ON_CORRUPTION|
+				 CRYPT_ACTIVATE_PANIC_ON_CORRUPTION|
 				 CRYPT_ACTIVATE_IGNORE_ZERO_BLOCKS|
 				 CRYPT_ACTIVATE_CHECK_AT_MOST_ONCE))
-			log_std("  flags:       %s%s%s%s\n",
+			log_std("  flags:       %s%s%s%s%s\n",
 				(cad.flags & CRYPT_ACTIVATE_IGNORE_CORRUPTION) ? "ignore_corruption " : "",
 				(cad.flags & CRYPT_ACTIVATE_RESTART_ON_CORRUPTION) ? "restart_on_corruption " : "",
+				(cad.flags & CRYPT_ACTIVATE_PANIC_ON_CORRUPTION) ? "panic_on_corruption " : "",
 				(cad.flags & CRYPT_ACTIVATE_IGNORE_ZERO_BLOCKS) ? "ignore_zero_blocks " : "",
 				(cad.flags & CRYPT_ACTIVATE_CHECK_AT_MOST_ONCE) ? "check_at_most_once" : "");
 	}
@@ -328,7 +451,7 @@ out:
 	return r;
 }
 
-static int action_dump(int arg)
+static int action_dump(void)
 {
 	struct crypt_device *cd = NULL;
 	struct crypt_params_verity params = {};
@@ -337,25 +460,31 @@ static int action_dump(int arg)
 	if ((r = crypt_init(&cd, action_argv[0])))
 		return r;
 
-	params.hash_area_offset = hash_offset;
-	params.fec_area_offset = fec_offset;
+	params.hash_area_offset = ARG_UINT64(OPT_HASH_OFFSET_ID);
+	params.fec_area_offset = ARG_UINT64(OPT_FEC_OFFSET_ID);
+	params.fec_device = ARG_STR(OPT_FEC_DEVICE_ID);
+	params.fec_roots = ARG_UINT32(OPT_FEC_ROOTS_ID);
+
 	r = crypt_load(cd, CRYPT_VERITY, &params);
 	if (!r)
 		crypt_dump(cd);
+	else
+		log_err(_("Device %s is not a valid VERITY device."), action_argv[0]);
+
 	crypt_free(cd);
 	return r;
 }
 
 static struct action_type {
 	const char *type;
-	int (*handler)(int);
+	int (*handler)(void);
 	int required_action_argc;
 	const char *arg_desc;
 	const char *desc;
 } action_types[] = {
 	{ "format",	action_format, 2, N_("<data_device> <hash_device>"),N_("format device") },
-	{ "verify",	action_verify, 3, N_("<data_device> <hash_device> <root_hash>"),N_("verify device") },
-	{ "open",	action_open,   4, N_("<data_device> <name> <hash_device> <root_hash>"),N_("open device as <name>") },
+	{ "verify",	action_verify, 2, N_("<data_device> <hash_device> [<root_hash>]"),N_("verify device") },
+	{ "open",	action_open,   3, N_("<data_device> <name> <hash_device> [<root_hash>]"),N_("open device as <name>") },
 	{ "close",	action_close,  1, N_("<name>"),N_("close device (remove mapping)") },
 	{ "status",	action_status, 1, N_("<name>"),N_("show active device status") },
 	{ "dump",	action_dump,   1, N_("<hash_device>"),N_("show on-disk information") },
@@ -371,7 +500,7 @@ static void help(poptContext popt_context,
 	struct action_type *action;
 
 	if (key->shortName == '?') {
-		log_std("%s %s\n", PACKAGE_VERITY, PACKAGE_VERSION);
+		tools_package_version(PACKAGE_VERITY, false);
 		poptPrintHelp(popt_context, stdout, 0);
 		log_std(_("\n"
 			 "<action> is one of:\n"));
@@ -390,10 +519,12 @@ static void help(poptContext popt_context,
 			DEFAULT_VERITY_HASH, DEFAULT_VERITY_DATA_BLOCK,
 			DEFAULT_VERITY_HASH_BLOCK, DEFAULT_VERITY_SALT_SIZE,
 			1);
+		tools_cleanup();
 		poptFreeContext(popt_context);
 		exit(EXIT_SUCCESS);
 	} else if (key->shortName == 'V') {
-		log_std("%s %s\n", PACKAGE_VERITY, PACKAGE_VERSION);
+		tools_package_version(PACKAGE_VERITY, false);
+		tools_cleanup();
 		poptFreeContext(popt_context);
 		exit(EXIT_SUCCESS);
 	} else
@@ -406,15 +537,31 @@ static int run_action(struct action_type *action)
 
 	log_dbg("Running command %s.", action->type);
 
-	r = action->handler(0);
+	r = action->handler();
 
 	show_status(r);
 	return translate_errno(r);
 }
 
+static void basic_options_cb(poptContext popt_context,
+		 enum poptCallbackReason reason __attribute__((unused)),
+		 struct poptOption *key,
+		 const char *arg,
+		 void *data __attribute__((unused)))
+{
+	tools_parse_arg_value(popt_context, tool_core_args[key->val].type, tool_core_args + key->val, arg, key->val, NULL);
+
+	switch (key->val) {
+	case OPT_DEBUG_ID:
+		log_parms.debug = true;
+		/* fall through */
+	case OPT_VERBOSE_ID:
+		log_parms.verbose = true;
+	}
+}
+
 int main(int argc, const char **argv)
 {
-	static char *popt_tmp;
 	static const char *null_action_argv[] = {NULL};
 	static struct poptOption popt_help_options[] = {
 		{ NULL,    '\0', POPT_ARG_CALLBACK, help, 0, NULL,                         NULL },
@@ -423,26 +570,16 @@ int main(int argc, const char **argv)
 		{ "version",'V', POPT_ARG_NONE,     NULL, 0, N_("Print package version"),  NULL },
 		POPT_TABLEEND
 	};
+	static struct poptOption popt_basic_options[] = {
+		{ NULL,    '\0', POPT_ARG_CALLBACK, basic_options_cb, 0, NULL, NULL },
+#define ARG(A, B, C, D, E, F, G, H) { A, B, C, NULL, A ## _ID, D, E },
+#include "veritysetup_arg_list.h"
+#undef ARG
+		POPT_TABLEEND
+	};
 	static struct poptOption popt_options[] = {
 		{ NULL,              '\0', POPT_ARG_INCLUDE_TABLE, popt_help_options, 0, N_("Help options:"), NULL },
-		{ "verbose",         'v',  POPT_ARG_NONE, &opt_verbose,      0, N_("Shows more detailed error messages"), NULL },
-		{ "debug",           '\0', POPT_ARG_NONE, &opt_debug,        0, N_("Show debug messages"), NULL },
-		{ "no-superblock",   0,    POPT_ARG_VAL,  &use_superblock,   0, N_("Do not use verity superblock"), NULL },
-		{ "format",          0,    POPT_ARG_INT,  &hash_type,        0, N_("Format type (1 - normal, 0 - original Chrome OS)"), N_("number") },
-		{ "data-block-size", 0,    POPT_ARG_INT,  &data_block_size,  0, N_("Block size on the data device"), N_("bytes") },
-		{ "hash-block-size", 0,    POPT_ARG_INT,  &hash_block_size,  0, N_("Block size on the hash device"), N_("bytes") },
-		{ "fec-roots",       0,    POPT_ARG_INT,  &fec_roots,        0, N_("FEC parity bytes"), N_("bytes") },
-		{ "data-blocks",     0,    POPT_ARG_STRING, &popt_tmp,       1, N_("The number of blocks in the data file"), N_("blocks") },
-		{ "fec-device",      0,    POPT_ARG_STRING, &fec_device,     0, N_("Path to device with error correction data"), N_("path") },
-		{ "hash-offset",     0,    POPT_ARG_STRING, &popt_tmp,       2, N_("Starting offset on the hash device"), N_("bytes") },
-		{ "fec-offset",      0,    POPT_ARG_STRING, &popt_tmp,       3, N_("Starting offset on the FEC device"), N_("bytes") },
-		{ "hash",            'h',  POPT_ARG_STRING, &hash_algorithm, 0, N_("Hash algorithm"), N_("string") },
-		{ "salt",            's',  POPT_ARG_STRING, &salt_string,    0, N_("Salt"), N_("hex string") },
-		{ "uuid",            '\0', POPT_ARG_STRING, &opt_uuid,       0, N_("UUID for device to use"), NULL },
-		{ "restart-on-corruption", 0,POPT_ARG_NONE,&opt_restart_on_corruption, 0, N_("Restart kernel if corruption is detected"), NULL },
-		{ "ignore-corruption", 0,  POPT_ARG_NONE, &opt_ignore_corruption,  0, N_("Ignore corruption, log it only"), NULL },
-		{ "ignore-zero-blocks", 0, POPT_ARG_NONE, &opt_ignore_zero_blocks, 0, N_("Do not verify zeroed blocks"), NULL },
-		{ "check-at-most-once", 0, POPT_ARG_NONE, &opt_check_at_most_once, 0, N_("Verify data block only the first time it is read"), NULL },
+		{ NULL,              '\0', POPT_ARG_INCLUDE_TABLE, popt_basic_options, 0, NULL, NULL },
 		POPT_TABLEEND
 	};
 
@@ -451,7 +588,7 @@ int main(int argc, const char **argv)
 	const char *aname;
 	int r;
 
-	crypt_set_log_callback(NULL, tool_log, NULL);
+	crypt_set_log_callback(NULL, tool_log, &log_parms);
 
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
@@ -461,32 +598,7 @@ int main(int argc, const char **argv)
 	poptSetOtherOptionHelp(popt_context,
 	                       _("[OPTION...] <action> <action-specific>"));
 
-	while((r = poptGetNextOpt(popt_context)) > 0) {
-		unsigned long long ull_value;
-		char *endp;
-
-		errno = 0;
-		ull_value = strtoull(popt_tmp, &endp, 10);
-		if (*endp || !*popt_tmp || !isdigit(*popt_tmp) ||
-		    (errno == ERANGE && ull_value == ULLONG_MAX) ||
-		    (errno != 0 && ull_value == 0))
-			r = POPT_ERROR_BADNUMBER;
-
-		switch(r) {
-			case 1:
-				data_blocks = ull_value;
-				break;
-			case 2:
-				hash_offset = ull_value;
-				break;
-			case 3:
-				fec_offset = ull_value;
-				break;
-		}
-
-		if (r < 0)
-			break;
-	}
+	while((r = poptGetNextOpt(popt_context)) > 0) {}
 
 	if (r < -1)
 		usage(popt_context, EXIT_FAILURE, poptStrerror(r),
@@ -529,34 +641,36 @@ int main(int argc, const char **argv)
 
 	if (action_argc < action->required_action_argc) {
 		char buf[128];
-		snprintf(buf, 128,_("%s: requires %s as arguments"), action->type, action->arg_desc);
+		if (snprintf(buf, 128,_("%s: requires %s as arguments"), action->type, action->arg_desc) < 0)
+			buf[0] = '\0';
 		usage(popt_context, EXIT_FAILURE, buf,
 		      poptGetInvocationName(popt_context));
 	}
 
-	if (data_block_size < 0 || hash_block_size < 0 || hash_type < 0) {
+	tools_check_args(action->type, tool_core_args, ARRAY_SIZE(tool_core_args), popt_context);
+
+	if (ARG_SET(OPT_IGNORE_CORRUPTION_ID) && ARG_SET(OPT_RESTART_ON_CORRUPTION_ID))
 		usage(popt_context, EXIT_FAILURE,
-		      _("Negative number for option not permitted."),
+		_("Option --ignore-corruption and --restart-on-corruption cannot be used together."),
+		poptGetInvocationName(popt_context));
+
+	if (ARG_SET(OPT_PANIC_ON_CORRUPTION_ID) && ARG_SET(OPT_RESTART_ON_CORRUPTION_ID))
+		usage(popt_context, EXIT_FAILURE,
+		_("Option --panic-on-corruption and --restart-on-corruption cannot be used together."),
+		poptGetInvocationName(popt_context));
+
+	if (ARG_SET(OPT_CANCEL_DEFERRED_ID) && ARG_SET(OPT_DEFERRED_ID))
+		usage(popt_context, EXIT_FAILURE,
+		      _("Options --cancel-deferred and --deferred cannot be used at the same time."),
 		      poptGetInvocationName(popt_context));
-	}
 
-	if ((opt_ignore_corruption || opt_restart_on_corruption || opt_ignore_zero_blocks) && strcmp(aname, "open"))
-		usage(popt_context, EXIT_FAILURE,
-		_("Option --ignore-corruption, --restart-on-corruption or --ignore-zero-blocks is allowed only for open operation.\n"),
-		poptGetInvocationName(popt_context));
-
-	if (opt_ignore_corruption && opt_restart_on_corruption)
-		usage(popt_context, EXIT_FAILURE,
-		_("Option --ignore-corruption and --restart-on-corruption cannot be used together.\n"),
-		poptGetInvocationName(popt_context));
-
-	if (opt_debug) {
-		opt_verbose = 1;
-		crypt_set_debug_level(-1);
+	if (ARG_SET(OPT_DEBUG_ID)) {
+		crypt_set_debug_level(CRYPT_DEBUG_ALL);
 		dbg_version_and_cmd(argc, argv);
 	}
 
 	r = run_action(action);
+	tools_cleanup();
 	poptFreeContext(popt_context);
 	return r;
 }
